/* FILE: colormap.h                 -*-Mode: c++-*-
 *
 * OOMMF colormap C++ class
 *
 */

#ifndef COLORMAP_H
#define COLORMAP_H

#include "nb.h"

/* End includes */     /* Optional directive to build.tcl */
//////////////////////////////////////////////////////////////////////////
// Color specification
//
class OommfPackedRGB {
private:
  unsigned char red,green,blue,pad;
public:
  OommfPackedRGB() : red(0), green(0), blue(0), pad(0) {}
  OommfPackedRGB(const char* tkcolorstr) { Set(tkcolorstr); }
  OommfPackedRGB(int r,int g,int b)
    : red((unsigned char)r), green((unsigned char)g),
      blue((unsigned char)b), pad(0) {}

  void Get(unsigned int &myred,unsigned int &mygreen,
	   unsigned int &myblue) const {
    myred=red; mygreen=green; myblue=blue;
  }

  void Set(const char* tkcolorstr);

  void Set(unsigned int new_red,unsigned int new_green,
	   unsigned int new_blue) {
    red   = (unsigned char)new_red;
    green = (unsigned char)new_green;
    blue  = (unsigned char)new_blue;
  }

  void Blend(REAL8m wgt1,OommfPackedRGB color1,
	     REAL8m wgt2,OommfPackedRGB color2);
  /// Generalization of Set, using wgt1*color1 + wgt2*color2.

  void Invert() {
    red   = (unsigned char)(255-red);
    green = (unsigned char)(255-green);
    blue  = (unsigned char)(255-blue);
  }

};

//////////////////////////////////////////////////////////////////////////
// Colormap class
//
#define TKCOLORLENGTH 6
class DisplayColorMap {
private:
  static const ClassDoc class_doc;
  int colorcount;  // Number of colors in colormap
  Nb_DString mapname;
  char **TkColor;       // For rapid access, store as both Tk color
  OommfPackedRGB *rgb;  // strings, and packed bytes.
  void Alloc(int _colorcount);
  void Free();
public:
  void Setup(int _colorcount,const char* _mapname,
	     const char* MakeColor(REAL4m relative_shade));
  DisplayColorMap(void);
  DisplayColorMap(int _colorcount,const char* _mapname,
		  const char* MakeColor(REAL4m relative_shade)) {
    Setup(_colorcount,_mapname,MakeColor);
  }
  DisplayColorMap(const DisplayColorMap&);
  DisplayColorMap& operator=(const DisplayColorMap&);
  ~DisplayColorMap() { Free(); }

  // Setup routines using predefined maps
  void Setup(int _colorcount,const char* _mapname);
  DisplayColorMap(int _colorcount,const char* _mapname) {
    Setup(_colorcount,_mapname);
  }

  const char* GetMapName() { return (const char*)mapname; }
  int GetColorCount() { return colorcount; }

  const char* GetColor(REAL4m relative_shade) const; // Given a floating
  /// point number between 0. and 1., returns a char* to the closest
  /// corresponding value in the colormap.  The returned pointer will be
  /// valid until the colormap is changed.

  // OommfPackedRGB access version of the last.  This one returns
  // a value instead of a pointer, however.
  OommfPackedRGB GetPackedColor(REAL4m relative_shade) const;
};

// The predefined MakeColor colormaps
const char* MakeColorRedBlackBlue(REAL4m relative_shade);
const char* MakeColorBlueWhiteRed(REAL4m relative_shade);
const char* MakeColorTealWhiteRed(REAL4m relative_shade);
const char* MakeColorBlackGrayWhite(REAL4m relative_shade);
const char* MakeColorBlackWhiteBlack(REAL4m relative_shade);
const char* MakeColorWhiteBlackWhite(REAL4m relative_shade);
const char* MakeColorWhiteGreenBlack(REAL4m relative_shade);
const char* MakeColorRedGreenBlueRed(REAL4m relative_shade);

#endif /* COLORMAP_H */
