/* FILE: demag.h            -*-Mode: c++-*-
 *
 * Average H demag field across rectangular cells.  This is a modified
 * version of the simpledemag class, which uses symmetries in the
 * interaction coefficients to reduce memory usage.
 *
 */

#ifndef _OXS_DEMAG
#define _OXS_DEMAG

#include "energy.h"
#include "fft.h"
#include "key.h"
#include "mesh.h"
#include "meshvalue.h"
#include "simstate.h"
#include "threevector.h"
#include "rectangularmesh.h"

/* End includes */

class Oxs_Demag:public Oxs_Energy {
private:
  mutable UINT4m rdimx,rdimy,rdimz;  // Natural size of real data
  mutable UINT4m cdimx,cdimy,cdimz; // Full size of complex data
  // 2*cdimx>=rdimx, cdimy>=rdimy, cdimz>=rdimz
  // cdim[xyz] should be powers of 2.
  mutable UINT4m cstridey,cstridez; // Strides across complex data
  // cstridey>=cdimx, cstridez>=cdimy*cstridey
  // The stride sizes for the real arrays are just double the
  // complex strides, except cstride1 and rstride1 are assumed
  // to be 1.  Total matrix size is effectively cdimz*cstridez
  // Oxs_Complex elements, or twice that many "double" elements.

  mutable UINT4m mesh_id;

  // The A## arrays hold demag coefficients, transformed into
  // frequency domain.  These are held long term.  xcomp,
  // ycomp, and zcomp are used as temporary space, first to hold
  // the transforms of Mx, My, and Mz, then to store Hx, Hy, and
  // Hz.
  mutable UINT4m adimx,adimy,adimz;
  mutable UINT4m astridey,astridez;
  mutable OXS_COMPLEX_REAL_TYPE *A00,*A01,*A02,*A11,*A12,*A22;
  mutable Oxs_Complex *xcomp,*ycomp,*zcomp;
  mutable Oxs_FFT3D fft; // All transforms are same size, so we need
  /// only one Oxs_FFT3D object.

  void FillCoefficientArrays(const Oxs_Mesh* mesh) const;

  UINT4m NextPowerOfTwo(UINT4m n) const;  // Helper function
  void ReleaseMemory();
public:
  virtual const char* ClassName() const; // ClassName() is
  /// automatically generated by the OXS_EXT_REGISTER macro.
  Oxs_Demag(const char* name,     // Child instance id
	    Oxs_Director* newdtr, // App director
	    Tcl_Interp* safe_interp, // Safe interpreter
	    const char* argstr);  // MIF input block parameters
  virtual ~Oxs_Demag() { ReleaseMemory(); }

  virtual void GetEnergyAndField(const Oxs_SimState& state,
                                 Oxs_MeshValue<REAL8m>& energy,
                                 Oxs_MeshValue<ThreeVector>& field
                                 ) const;
};


#endif // _OXS_DEMAG
