/* FILE: sectionatlas.h                 -*-Mode: c++-*-
 *
 * Section atlas class, derived from Oxs_Atlas class.
 *
 */

#ifndef _OXS_SECTIONATLAS
#define _OXS_SECTIONATLAS

#include <string>
#include <vector>

#include "ext.h"
#include "section.h"
#include "threevector.h"
#include "util.h"
#include "atlas.h"
/* End includes */

class Oxs_SectionAtlas:public Oxs_Atlas {
private:
  vector<Oxs_Section*> section_list;  // Ordered list of sections
  vector<string> section_name_list; // Corresponding section names
public:
  virtual const char* ClassName() const; // ClassName() is
  /// automatically generated by the OXS_EXT_REGISTER macro.
  virtual BOOL Init() { return 1; }

  Oxs_SectionAtlas(const char* name,
		   Oxs_Director* newdtr,
		   Tcl_Interp* safe_interp, // Safe interpreter
		   const char* argstr);   // MIF block argument string

  ~Oxs_SectionAtlas();

  void GetWorldExtents(Oxs_Box &mybox) const;
  /// Fills mybox with bounding box for the atlas

  INT4m GetRegionId(const ThreeVector& point) const;
  /// Returns the id number for the first region in
  /// "section_list" containing point.  The return
  /// value is -1 if the point is not contained in
  /// the atlas.

  INT4m GetRegionId(const string& name) const;
  /// Given a region id string (name), returns
  /// the corresponding region id index.  If
  /// "name" is not included in the atlas, then
  /// -1 is returned.

  INT4m GetRegionName(UINT4m id,string& name) const;
  /// Given an id number, fills in "name" with
  /// the corresponding region id string.  Returns
  /// 0 on success, 1 if id is invalid.

  UINT4m GetRegionCount() const { return UINT4m(section_list.size()); }
  /// Valid RegionId numbers range from 0 to GetRegionCount -1,
  /// inclusive.

};

#endif // _OXS_SECTIONATLAS
